==> down.php?SDK=iOS_V1.20.2.3.zip: iOS_V1.20.2.3/Sample/Source/AuthV4Helper/AuthV4HelperViewController.swift <==
//
//  AuthV4HelperViewController
//  HIVE_SDK_Sample
//
//  Created by jhjang on 03/12/2018.
//  Copyright © 2018 gcp. All rights reserved.
//

import UIKit
import HIVEService

enum AuthV4HelperApi: String, CaseIterable {
    case setup = "[AuthV4] Setup"
    case signIn = "SignIn"
    case signOut = "SignOut"
    case connect = "Connect"
    case disconnect = "Disconnect"
    case syncAccount = "SyncAccount"
    case showAchievements = "Show Achievements"
    case showLeaderboard = "Show Leaderboard"
    case showConflict = "Show Conflict"
    case showConflictGame = "Show Conflict Game"
    case resolveConflict = "Resolve Conflict"
    case switchAccount = "Switch Account"
    case showGameCenterCancelDialog = "Show GameCenter Cancel Dialog"
    
    case showSignIn = "[AuthV4] Show SignIn"
    case reset = "[AuthV4] Reset"
}

enum AuthV4HelperIDPType {
    case connect
    case disconnect
}

class AuthV4HelperViewController: BaseViewController {
    let apis: [AuthV4HelperApi] = AuthV4HelperApi.allCases
    var providerList: [ProviderType]?
    var actionSheetType: AuthV4HelperIDPType!
    var conflictPlayer: PlayerInfo?
    
    var actionSheet: UIAlertController {
        let actionSheet = UIAlertController(title: "Provider", message: "Select Provider", preferredStyle: .actionSheet)
        
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        actionSheet.addAction(cancelAction)
        
        if let providerList = self.providerList {
            for item in providerList {
                let actionStyle:UIAlertAction.Style = item == .Auto ? .destructive : .default
                
                let action = UIAlertAction(title: item.description, style: actionStyle) { (action) in
                    switch self.actionSheetType! {
                    case .connect:
                        AuthV4Interface.helper().connect(item) { (result, playerInfo) in
                            let log = """
                            === AuthV4HelperInterface.connect() ===
                            
                            result - \(result)
                            playerInfo - \(String(describing: playerInfo))
                            """
                            
                            self.logDelegate?.addLog(log: log)
                            
                            if result.getErrorCode() == .conflictPlayer {
                                self.conflictPlayer = playerInfo
                            }
                        }
                    case .disconnect:
                        AuthV4Interface.helper().disconnect(item, handler: { (result, playerInfo) in
                            let log = """
                            === AuthV4HelperInterface.disconnect() ===
                            
                            result - \(result)
                            """
                            
                            self.logDelegate?.addLog(log: log)
                        })
                    }
                }
                actionSheet.addAction(action)
            }
        }
        
        return actionSheet
    }
    
    @IBOutlet weak var tableView: UITableView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.tableView.contentInset = UIEdgeInsets.init(top: 44, left: 0, bottom: 0, right: 0)
    }
    
    func callAPI(api: AuthV4HelperApi) {
        switch api {
        case .setup:
            AuthV4Interface.setup { (result, isAutoSignIn, did, providerList) in
                let log = """
                === AuthV4Helper.setup() ===
                
                result - \(result)
                isAutoSignIn - \(isAutoSignIn)
                did - \(String(describing: did))
                providerList - \(String(describing: providerList))
                """
                
                self.logDelegate?.addLog(log: log)
                
                self.providerList = providerList?.compactMap { ProviderType(rawValue: $0) }
                
                if isAutoSignIn {
                    self.providerList?.insert(.Auto, at: 0)
                }
            }
        case .signIn:
            AuthV4Interface.helper().signIn { (result, playerInfo) in
                let log = """
                === AuthV4Helper.signIn() ===
                
                result - \(result)
                playerInfo - \(String(describing: playerInfo))
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .signOut:
            AuthV4Interface.helper().signOut { (result, playerInfo) in
                let log = """
                === AuthV4Helper.signIn() ===
                
                result - \(result)
                playerInfo - \(String(describing: playerInfo))
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .connect:
            self.actionSheetType = .connect
            self.present(self.actionSheet, animated: true, completion: nil)
        case .disconnect:
            self.actionSheetType = .disconnect
            self.present(self.actionSheet, animated: true, completion: nil)
        case .syncAccount:
            AuthV4Interface.helper().syncAccount(.Apple) { (result, playerInfo) in
                let log = """
                === AuthV4Helper.syncAccount() ===
                
                result - \(result)
                playerInfo - \(String(describing: playerInfo))
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .showAchievements:
            AuthV4Interface.helper().showAchievements { (result, playerInfo) in
                let log = """
                === AuthV4Helper.showAchievements() ===
                
                result - \(result)
                playerInfo - \(String(describing: playerInfo))
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .showLeaderboard:
            AuthV4Interface.helper().showLeaderboard { (result, playerInfo) in
                let log = """
                === AuthV4Helper.showLeaderboard() ===
                
                result - \(result)
                playerInfo - \(playerInfo)
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .showConflict:
            AuthV4Interface.helper().showConflict { (result, playerInfo) in
                let log = """
                === AuthV4Helper.showConflict() ===
                
                result - \(result)
                playerInfo - \(String(describing: playerInfo))
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .showConflictGame:
            let viewInfo = ConflictSingleViewInfo()
            viewInfo.setValue("Gold", intValue: 2048)
            viewInfo.setValue("Gem", intValue: 220)
            
            AuthV4Interface.helper().showConflict(viewInfo) { (result, playerInfo) in
                let log = """
                === AuthV4Helper.showConflictGame() ===
                
                result - \(result)
                playerInfo - \(String(describing: playerInfo))
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .resolveConflict:
            AuthV4Interface.helper().resolveConflict { (result, playerInfo) in
                let log = """
                === AuthV4Helper.resolveConflict() ===
                
                result - \(result)
                playerInfo - \(String(describing: playerInfo))
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .switchAccount:
            AuthV4Interface.helper().switchAccount { (result, playerInfo) in
                let log = """
                === AuthV4Helper.switchAccount() ===
                
                result - \(result)
                playerInfo - \(String(describing: playerInfo))
                """
                
                self.logDelegate?.addLog(log: log)
            }
            
        case .showSignIn:
            AuthV4Interface.showSignIn { (result, playerInfo) in
                let log = """
                === AuthV4Helper.showSignIn() ===
                
                result - \(result)
                playerInfo - \(String(describing: playerInfo))
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .showGameCenterCancelDialog:
            AuthV4Interface.helper().showGameCenterLoginCancelDialog { (finished) in
                let log = """
                === AuthV4Helper.showGameCenterCancelDialog() ===

                finished - \(finished)
                """

                self.logDelegate?.addLog(log: log)
            }
        case .reset:
            AuthV4Interface.reset()
        }
    }
    
    private func showConflict() {
//        if self.conflictPlayer != nil {
//            let alertViewController = UIAlertController(title: "Conflict", message: "select", preferredStyle: .alert)
//
//            let selectAction = UIAlertAction(title: "switch", style: .cancel) { _ in
//                AuthV4HelperInterface.selectConflict(self.conflictPlayer!.playerID, handler: { (result, playerInfo) in
//                    let log = """
//                    === AuthV4HelperInterface.selectConflict() ===
//
//                    result - \(result)
//                    playerInfo - \(playerInfo)
//                    """
//
//                    self.logDelegate?.addLog(log: log)
//                })
//            }
//
//            let resolveAction = UIAlertAction(title: "resolve", style: .default) { (result) in
//                AuthV4HelperInterface.resolveConflict({ (result) in
//                    let log = """
//                    === AuthV4HelperInterface.resolveConflict() ===
//
//                    result - \(result)
//                    """
//
//                    self.logDelegate?.addLog(log: log)
//                })
//            }
//
//            alertViewController.addAction(selectAction)
//            alertViewController.addAction(resolveAction)
//
//            self.present(alertViewController, animated: true, completion: nil)
//        }else {
//            Logger.d("Conflict info is nil.")
//        }
    }
}

extension AuthV4HelperViewController: UITableViewDelegate, UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return self.apis.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard let cell = tableView.dequeueReusableCell(withIdentifier: "ApiCell") as? AuthV4HelperTableViewCell else {
            preconditionFailure("Not supported cell type")
        }
        
        cell.configureCell(api: self.apis[indexPath.row])
        
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        self.callAPI(api: apis[indexPath.row])
        tableView.deselectRow(at: indexPath, animated: true)
    }
}

==> down.php?SDK=iOS_V1.20.3.2.zip: iOS_V1.20.3.2/Sample/Source/AuthV4Helper/AuthV4HelperViewController.swift <==
//
//  AuthV4HelperViewController
//  HIVE_SDK_Sample
//
//  Created by jhjang on 03/12/2018.
//  Copyright © 2018 gcp. All rights reserved.
//

import UIKit
import HIVEService

enum AuthV4HelperApi: String, CaseIterable {
    case setup = "[AuthV4] Setup"
    case signIn = "SignIn"
    case signOut = "SignOut"
    case connect = "Connect"
    case disconnect = "Disconnect"
    case syncAccount = "SyncAccount"
    case showAchievements = "Show Achievements"
    case showLeaderboard = "Show Leaderboard"
    case showConflict = "Show Conflict"
    case showConflictGame = "Show Conflict Game"
    case resolveConflict = "Resolve Conflict"
    case switchAccount = "Switch Account"
    case showGameCenterCancelDialog = "Show GameCenter Cancel Dialog"
    
    case showSignIn = "[AuthV4] Show SignIn"
    case reset = "[AuthV4] Reset"
}

enum AuthV4HelperIDPType {
    case connect
    case disconnect
}

class AuthV4HelperViewController: BaseViewController {
    let apis: [AuthV4HelperApi] = AuthV4HelperApi.allCases
    var providerList: [ProviderType]?
    var actionSheetType: AuthV4HelperIDPType!
    var conflictPlayer: PlayerInfo?
    
    var actionSheet: UIAlertController {
        let actionSheet = UIAlertController(title: "Provider", message: "Select Provider", preferredStyle: .actionSheet)
        
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        actionSheet.addAction(cancelAction)
        
        if let providerList = self.providerList {
            for item in providerList {
                let actionStyle: UIAlertAction.Style = item == .Auto ? .destructive : .default
                
                let action = UIAlertAction(title: item.description, style: actionStyle) { (_) in
                    switch self.actionSheetType! {
                    case .connect:
                        AuthV4Interface.helper().connect(item) { (result, playerInfo) in
                            let log = """
                            === AuthV4HelperInterface.connect() ===
                            
                            result - \(result)
                            playerInfo - \(String(describing: playerInfo))
                            """
                            
                            self.logDelegate?.addLog(log: log)
                            
                            if result.getErrorCode() == .conflictPlayer {
                                self.conflictPlayer = playerInfo
                            }
                        }
                    case .disconnect:
                        AuthV4Interface.helper().disconnect(item, handler: { (result, _) in
                            let log = """
                            === AuthV4HelperInterface.disconnect() ===
                            
                            result - \(result)
                            """
                            
                            self.logDelegate?.addLog(log: log)
                        })
                    }
                }
                actionSheet.addAction(action)
            }
        }
        
        return actionSheet
    }
    
    @IBOutlet weak var tableView: UITableView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.tableView.contentInset = UIEdgeInsets.init(top: 44, left: 0, bottom: 0, right: 0)
    }
    
    func callAPI(api: AuthV4HelperApi) {
        switch api {
        case .setup:
            AuthV4Interface.setup { (result, isAutoSignIn, did, providerList) in
                let log = """
                === AuthV4Helper.setup() ===
                
                result - \(result)
                isAutoSignIn - \(isAutoSignIn)
                did - \(String(describing: did))
                providerList - \(String(describing: providerList))
                """
                
                self.logDelegate?.addLog(log: log)
                
                self.providerList = providerList?.compactMap { ProviderType(rawValue: $0) }
                
                if isAutoSignIn {
                    self.providerList?.insert(.Auto, at: 0)
                }
            }
        case .signIn:
            AuthV4Interface.helper().signIn { (result, playerInfo) in
                let log = """
                === AuthV4Helper.signIn() ===
                
                result - \(result)
                playerInfo - \(String(describing: playerInfo))
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .signOut:
            AuthV4Interface.helper().signOut { (result, playerInfo) in
                let log = """
                === AuthV4Helper.signIn() ===
                
                result - \(result)
                playerInfo - \(String(describing: playerInfo))
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .connect:
            self.actionSheetType = .connect
            self.present(self.actionSheet, animated: true, completion: nil)
        case .disconnect:
            self.actionSheetType = .disconnect
            self.present(self.actionSheet, animated: true, completion: nil)
        case .syncAccount:
            AuthV4Interface.helper().syncAccount(.Apple) { (result, playerInfo) in
                let log = """
                === AuthV4Helper.syncAccount() ===
                
                result - \(result)
                playerInfo - \(String(describing: playerInfo))
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .showAchievements:
            AuthV4Interface.helper().showAchievements { (result, playerInfo) in
                let log = """
                === AuthV4Helper.showAchievements() ===
                
                result - \(result)
                playerInfo - \(String(describing: playerInfo))
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .showLeaderboard:
            AuthV4Interface.helper().showLeaderboard { (result, playerInfo) in
                let log = """
                === AuthV4Helper.showLeaderboard() ===
                
                result - \(result)
                playerInfo - \(playerInfo?.description ?? "Null Instance")
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .showConflict:
            AuthV4Interface.helper().showConflict { (result, playerInfo) in
                let log = """
                === AuthV4Helper.showConflict() ===
                
                result - \(result)
                playerInfo - \(String(describing: playerInfo))
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .showConflictGame:
            let viewInfo = ConflictSingleViewInfo()
            viewInfo.setValue("Gold", intValue: 2048)
            viewInfo.setValue("Gem", intValue: 220)
            
            AuthV4Interface.helper().showConflict(viewInfo) { (result, playerInfo) in
                let log = """
                === AuthV4Helper.showConflictGame() ===
                
                result - \(result)
                playerInfo - \(String(describing: playerInfo))
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .resolveConflict:
            AuthV4Interface.helper().resolveConflict { (result, playerInfo) in
                let log = """
                === AuthV4Helper.resolveConflict() ===
                
                result - \(result)
                playerInfo - \(String(describing: playerInfo))
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .switchAccount:
            AuthV4Interface.helper().switchAccount { (result, playerInfo) in
                let log = """
                === AuthV4Helper.switchAccount() ===
                
                result - \(result)
                playerInfo - \(String(describing: playerInfo))
                """
                
                self.logDelegate?.addLog(log: log)
            }
            
        case .showSignIn:
            AuthV4Interface.showSignIn { (result, playerInfo) in
                let log = """
                === AuthV4Helper.showSignIn() ===
                
                result - \(result)
                playerInfo - \(String(describing: playerInfo))
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .showGameCenterCancelDialog:
            AuthV4Interface.helper().showGameCenterLoginCancelDialog { (finished) in
                let log = """
                === AuthV4Helper.showGameCenterCancelDialog() ===

                finished - \(finished)
                """

                self.logDelegate?.addLog(log: log)
            }
        case .reset:
            AuthV4Interface.reset()
        }
    }
    
    private func showConflict() {
//        if self.conflictPlayer != nil {
//            let alertViewController = UIAlertController(title: "Conflict", message: "select", preferredStyle: .alert)
//
//            let selectAction = UIAlertAction(title: "switch", style: .cancel) { _ in
//                AuthV4HelperInterface.selectConflict(self.conflictPlayer!.playerID, handler: { (result, playerInfo) in
//                    let log = """
//                    === AuthV4HelperInterface.selectConflict() ===
//
//                    result - \(result)
//                    playerInfo - \(playerInfo)
//                    """
//
//                    self.logDelegate?.addLog(log: log)
//                })
//            }
//
//            let resolveAction = UIAlertAction(title: "resolve", style: .default) { (result) in
//                AuthV4HelperInterface.resolveConflict({ (result) in
//                    let log = """
//                    === AuthV4HelperInterface.resolveConflict() ===
//
//                    result - \(result)
//                    """
//
//                    self.logDelegate?.addLog(log: log)
//                })
//            }
//
//            alertViewController.addAction(selectAction)
//            alertViewController.addAction(resolveAction)
//
//            self.present(alertViewController, animated: true, completion: nil)
//        }else {
//            Logger.d("Conflict info is nil.")
//        }
    }
}

extension AuthV4HelperViewController: UITableViewDelegate, UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        self.apis.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard let cell = tableView.dequeueReusableCell(withIdentifier: "ApiCell") as? AuthV4HelperTableViewCell else {
            preconditionFailure("Not supported cell type")
        }
        
        cell.configureCell(api: self.apis[indexPath.row])
        
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        self.callAPI(api: apis[indexPath.row])
        tableView.deselectRow(at: indexPath, animated: true)
    }
}
